@@ -1,7 +1,8 @@
 package CPAN::Reporter;
 use strict;
-
-$CPAN::Reporter::VERSION = '1.0602'; 
+use vars qw/$VERSION/;
+$VERSION = '1.08'; 
+$VERSION = eval $VERSION;
 
 use Config;
 use CPAN ();
@@ -11,7 +12,6 @@ use File::Find ();
 use File::HomeDir ();
 use File::Path qw/mkpath rmtree/;
 use File::Spec ();
-use File::Temp 0.16 ();
 use IO::File ();
 use Probe::Perl ();
 use Tee qw/tee/;
@@ -32,14 +32,16 @@ sub configure {
 
 sub grade_make { 
     my @args = @_;
-    my $result = _init_result( 'make', @args ); 
+    my $result = _init_result( 'make', @args ) or return; 
     _compute_make_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->myprint( 
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
-            "Test results for $result->{dist_name} will be discarded"
+            "Test report will not be sent"
         );
+        CPAN::Reporter::History::_record_history( $result ) 
+            if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
         _print_grade_msg($result->{make_cmd}, $result);
@@ -50,14 +52,16 @@ sub grade_make {
 
 sub grade_PL {
     my @args = @_;
-    my $result = _init_result( 'PL', @args ); ## no critic
+    my $result = _init_result( 'PL', @args ) or return; 
     _compute_PL_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->myprint( 
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
-            "Test results for $result->{dist_name} will be discarded"
+            "Test report will not be sent"
         );
+        CPAN::Reporter::History::_record_history( $result ) 
+            if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
         _print_grade_msg($result->{PL_file} , $result);
@@ -68,14 +72,16 @@ sub grade_PL {
 
 sub grade_test {
     my @args = @_;
-    my $result = _init_result( 'test', @args ); ## no critic
+    my $result = _init_result( 'test', @args ) or return; 
     _compute_test_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->myprint( 
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
-            "Test results for $result->{dist_name} will be discarded"
+            "Test report will not be sent"
         );
+        CPAN::Reporter::History::_record_history( $result ) 
+            if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
         _print_grade_msg( "Test", $result );
@@ -87,18 +93,21 @@ sub grade_test {
 sub record_command {
     my ($command, $timeout) = @_;
 
+    # XXX refactor this! 
+    # Get configuration options
+    my $config_obj = CPAN::Reporter::Config::_open_config_file();
+    my $config;
+    $config = CPAN::Reporter::Config::_get_config_options( $config_obj ) 
+        if $config_obj;
+
+    $timeout ||= $config->{command_timeout}; # might still be undef
+
     my ($cmd, $redirect) = _split_redirect($command);
 
-    my $temp_out = File::Temp->new( 
-        TEMPLATE => 'CPAN-Reporter-TO-XXXXXXXX', DIR => File::Spec->tmpdir()
-    ) or die "Could not create a temporary file for output: $!";
+    my $temp_out = _temp_filename( 'CPAN-Reporter-TO-' );
 
     # Teeing a command loses its exit value so we must wrap the command 
     # and print the exit code so we can read it off of output
-    my $cmdwrapper = File::Temp->new( 
-        TEMPLATE => 'CPAN-Reporter-CW-XXXXXXXX', DIR => File::Spec->tmpdir() 
-    ) or die "Could not create a wrapper for $cmd\: $!";
-
     my $wrap_code;
     if ( $timeout ) {
         $wrap_code = $^O eq 'MSWin32'
@@ -115,23 +124,33 @@ print "($safecmd exited with \$ec)\\n";
 HERE
     }
 
-    print {$cmdwrapper} $wrap_code;
-    $cmdwrapper->close;
+    # write code to a tempfile for execution
+    my $wrapper_name = _temp_filename( 'CPAN-Reporter-CW-' );
+    my $wrapper_fh = IO::File->new( $wrapper_name, 'w' )
+        or die "Could not create a wrapper for $cmd\: $!";
+
+    $wrapper_fh->print( $wrap_code );
+    $wrapper_fh->close;
     
     # tee the command wrapper
-    my $tee_input = Probe::Perl->find_perl_interpreter() .  " $cmdwrapper";
+    my $tee_input = Probe::Perl->find_perl_interpreter() .  " $wrapper_name";
     $tee_input .= " $redirect" if defined $redirect;
     tee($tee_input, { stderr => 1 }, $temp_out);
         
     # read back the output
-    my $temp_out2 = IO::File->new($temp_out->filename, "<");
-    if ( !$temp_out2 ) {
+    my $output_fh = IO::File->new($temp_out, "r");
+    if ( !$output_fh ) {
         $CPAN::Frontend->mywarn( 
             "CPAN::Reporter: couldn't read command results for '$cmd'\n" 
         );
         return;
     }
-    my @cmd_output = <$temp_out2>;
+    my @cmd_output = <$output_fh>;
+    $output_fh->close;
+
+    # cleanup
+    unlink $wrapper_name, $temp_out;
+
     if ( ! @cmd_output ) {
         $CPAN::Frontend->mywarn( 
             "CPAN::Reporter: didn't capture command results for '$cmd'\n"
@@ -158,12 +177,6 @@ HERE
 sub test {
     my ($dist, $system_command) = @_;
     my ($output, $exit_value) = record_command( $system_command );
-    unless ( defined $output && defined $exit_value ) {
-        $CPAN::Frontend->mywarn(
-            "CPAN::Reporter: had errors capturing output. Tests abandoned"
-        );
-        return;
-    }
     return grade_test( $dist, $system_command, $output, $exit_value );
 }
 
@@ -347,6 +360,28 @@ END_BAD_DISTNAME
     # Gather 'expensive' data for the report
     _expand_result( $result);
 
+    # Skip if distribution name matches the send_skipfile
+    if ( $config->{send_skipfile} && -r $config->{send_skipfile} ) {
+        my $send_skipfile = IO::File->new( $config->{send_skipfile}, "r" );
+        my $dist_id = $result->{dist}->pretty_id;
+        while ( my $pattern = <$send_skipfile> ) {
+            chomp($pattern);
+            # ignore comments
+            next if substr($pattern,0,1) eq '#';
+            # if it doesn't match, continue with next pattern
+            next if $dist_id !~ /$pattern/;
+            # if it matches, warn and return
+            $CPAN::Frontend->myprint( << "END_SKIP_DIST" );
+CPAN::Reporter: '$dist_id' matched against the send_skipfile.  
+
+Test report will not be sent.
+
+END_SKIP_DIST
+
+            return;
+        }
+    }
+
     # Setup the test report
     my $tr = Test::Reporter->new;
     $tr->grade( $result->{grade} );
@@ -356,11 +391,10 @@ END_BAD_DISTNAME
     my $is_duplicate = CPAN::Reporter::History::_is_duplicate( $result );
     if ( $is_duplicate ) {
         if ( _prompt( $config, "send_duplicates", $tr->grade) =~ /^n/ ) {
-            $CPAN::Frontend->mywarn(<< "DUPLICATE_REPORT");
+            $CPAN::Frontend->myprint(<< "DUPLICATE_REPORT");
 
-CPAN::Reporter: it seems that "@{[$tr->subject]}"
-during the $phase phase is a duplicate of a previous report you 
-sent to CPAN Testers.
+CPAN::Reporter: this appears to be a duplicate report for the $phase phase:
+@{[$tr->subject]}
 
 Test report will not be sent.
 
@@ -393,19 +427,8 @@ DUPLICATE_REPORT
     # Populate the test report
     $tr->comments( _report_text( $result ) );
     $tr->via( 'CPAN::Reporter ' . $CPAN::Reporter::VERSION );
-    my @cc;
+    my @cc = _should_copy_author( $result, $config );
 
-    # User prompts for action
-    my $author_email = $result->{author_id} 
-                     ? "$result->{author_id}\@cpan.org"
-                     : q{};
-    if ( ! $author_email ) {
-        $CPAN::Frontend->mywarn( "CPAN::Reporter: couldn't determine author_id and won't cc author.\n");
-    }
-    if ( $author_email && _prompt( $config, "cc_author", $tr->grade, "($author_email)?") =~ /^y/ ) {
-        push @cc, $author_email;
-    }
-    
     # prompt for editing report
     if ( _prompt( $config, "edit_report", $tr->grade ) =~ /^y/ ) {
         my $editor = $config->{editor};
@@ -448,12 +471,21 @@ sub _downgrade_known_causes {
     my ($grade, $output) = ( $result->{grade}, $result->{output} );
     my $msg = $result->{grade_msg} || q{};
 
-    # shortcut unless fail/unknown
-    return if $grade eq 'pass' || $grade eq 'na';
+    # shortcut unless fail/unknown; but PL might look like pass but actually
+    # have "OS Unsupported" messages
+    return if $grade eq 'na';
+    return if $grade eq 'pass' && $result->{phase} ne 'PL';
 
     # get prereqs
     _expand_result( $result ); 
 
+    # if process was halted with a signal, just set for discard and return
+    if ( $result->{exit_value} & 127 ) {
+        $result->{grade} = 'discard';
+        $result->{grade_msg} = 'Command interrupted';
+        return;
+    }
+
     # look for perl version error messages from various programs
     # "Error evaling..." type errors happen on Perl < 5.006 when modules
     # define their version with "our $VERSION = ..."
@@ -557,17 +589,6 @@ sub _env_report {
     return $report;
 }
 
-#--------------------------------------------------------------------------#
-# _format_distname
-#--------------------------------------------------------------------------#
-
-sub _format_distname {
-    my $dist = shift;
-    my $basename = basename( $dist->pretty_id );
-    $basename =~ s/(\.tar\.(?:gz|bz2)|\.tgz|\.zip)$//i;
-    return $basename;
-}
-
 #--------------------------------------------------------------------------#
 # _has_recursive_make
 #
@@ -600,6 +621,13 @@ sub _has_recursive_make {
 sub _init_result {
     my ($phase, $dist, $system_command, $output, $exit_value) = @_;
     
+    unless ( defined $output && defined $exit_value ) {
+        $CPAN::Frontend->mywarn(
+            "CPAN::Reporter: had errors capturing output. Tests abandoned"
+        );
+        return;
+    }
+
     my $result = {
         phase => $phase,
         dist => $dist,
@@ -609,7 +637,7 @@ sub _init_result {
         exit_value => $exit_value,
         # Note: pretty_id is like "DAGOLDEN/CPAN-Reporter-0.40.tar.gz"
         dist_basename => basename($dist->pretty_id),
-        dist_name => _format_distname( $dist ),
+        dist_name => $dist->base_id,
     };
 
     # Used in messages to user
@@ -925,6 +953,53 @@ ENDREPORT
     return $output;
 }
 
+
+#--------------------------------------------------------------------------#
+# _should_copy_author
+#--------------------------------------------------------------------------#
+
+sub _should_copy_author {
+    my ($result, $config) = @_;
+
+    # User prompts for action
+    my $author_email = $result->{author_id} 
+                     ? "$result->{author_id}\@cpan.org"
+                     : q{};
+    if ( ! $author_email ) {
+        $CPAN::Frontend->mywarn( "CPAN::Reporter: couldn't determine author_id and won't cc author.\n");
+        return;
+    }
+
+    # Skip if distribution name matches the cc_skipfile
+    if ( $config->{cc_skipfile} && -r $config->{cc_skipfile} ) {
+        my $cc_skipfile = IO::File->new( $config->{cc_skipfile}, "r" );
+        my $dist_id = $result->{dist}->pretty_id;
+        while ( my $pattern = <$cc_skipfile> ) {
+            chomp($pattern);
+            # ignore comments
+            next if substr($pattern,0,1) eq '#';
+            # if it doesn't match, continue with next pattern
+            next if $dist_id !~ /$pattern/;
+            # if it matches, warn and return
+            $CPAN::Frontend->myprint( << "END_SKIP_DIST" );
+CPAN::Reporter: '$dist_id' matched against the cc_skipfile.  Won't copy author.
+END_SKIP_DIST
+            return;
+        }
+    }
+
+    # Don't copy author on perls with patchlevels
+    return if $Config{perl_patchlevel};
+
+    # Finally, prompt user if necessary
+    if ( _prompt( $config, "cc_author", $result->{grade}, "($author_email)?") =~ /^y/ ) {
+        return $author_email;
+    }
+    else {
+        return;
+    }
+}
+
 #--------------------------------------------------------------------------#
 # _special_vars_report
 #--------------------------------------------------------------------------#
@@ -961,6 +1036,28 @@ sub _split_redirect {
     }
 }
 
+#--------------------------------------------------------------------------#
+# _temp_filename -- stand-in for File::Temp for backwards compatibility
+#
+# takes an optional prefix, adds 8 random chars and returns 
+# an absolute pathname
+#
+# NOTE -- manual unlink required
+#--------------------------------------------------------------------------#
+
+# @CHARS from File::Temp
+my @CHARS = (qw/ A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
+                 a b c d e f g h i j k l m n o p q r s t u v w x y z
+                 0 1 2 3 4 5 6 7 8 9 _
+             /);
+
+sub _temp_filename {
+    my ($prefix) = @_;
+    $prefix = q{} unless defined $prefix;
+    $prefix .= $CHARS[ int( rand(@CHARS) ) ] for 0 .. 7;
+    return File::Spec->catfile(File::Spec->tmpdir(), $prefix);
+}
+
 #--------------------------------------------------------------------------#
 # _timeout_wrapper
 #--------------------------------------------------------------------------#
@@ -984,12 +1081,13 @@ eval {
         my $wstat = waitpid $pid, 0;
         $exitcode = $wstat == -1 ? -1 : $?;
     } else {    #child
+        setpgrp; # new process group for targeted kill
         exec "%s";
     }
 };
 alarm 0;
 if ($pid && $@ =~ /Timeout/){
-    kill 9, $pid;
+    kill -9, $pid;
     my $wstat = waitpid $pid, 0;
     $exitcode = $wstat == -1 ? -1 : $?;
 }
@@ -1131,14 +1229,16 @@ sub _version_finder {
     my $perl = Probe::Perl->find_perl_interpreter();
     my @prereq_results;
     
-    my $prereq_input = File::Temp->new( 
-        TEMPLATE => 'CPAN-Reporter-PI-XXXXXXXX', DIR => File::Spec->tmpdir() 
-    ) or die "Could not create temporary input for prereq analysis: $!";
-    $prereq_input->print( map { "$_ $prereqs{$_}\n" } keys %prereqs );
-    $prereq_input->close;
+    my $prereq_input = _temp_filename( 'CPAN-Reporter-PI-' );
+    my $fh = IO::File->new( $prereq_input, "w" )
+        or die "Could not create temporary '$prereq_input' for prereq analysis: $!";
+    $fh->print( map { "$_ $prereqs{$_}\n" } keys %prereqs );
+    $fh->close;
 
     my $prereq_result = qx/$perl $version_finder < $prereq_input/;
 
+    unlink $prereq_input;
+
     my %result;
     for my $line ( split "\n", $prereq_result ) {
         my ($mod, $met, $have) = split " ", $line;
@@ -1319,7 +1419,7 @@ David A. Golden (DAGOLDEN)
 
 = COPYRIGHT AND LICENSE
 
-Copyright (c) 2006, 2007 by David A. Golden
+Copyright (c) 2006, 2007, 2008 by David A. Golden
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
