@@ -1,7 +1,8 @@
 package CPAN::Reporter::Config;
 use strict; 
 use vars qw/$VERSION/;
-$VERSION = '1.0602'; 
+$VERSION = '1.08'; 
+$VERSION = eval $VERSION;
 
 use Config::Tiny ();
 use File::HomeDir (); 
@@ -218,12 +219,12 @@ HERE
     },
     edit_report => {
         default => 'default:ask/no pass/na:no',
-        prompt => "Do you want to edit the test report?",
+        prompt => "Do you want to review or edit the test report?",
         validate => \&_validate_grade_action_pair,
         info => <<'HERE',
-Before test reports are sent, you may want to edit the test report
-and add additional comments about the result or about your system or
-Perl configuration.  By default, CPAN::Reporter will ask after
+Before test reports are sent, you may want to review or edit the test 
+report and add additional comments about the result or about your system 
+or Perl configuration.  By default, CPAN::Reporter will ask after
 each report is generated whether or not you would like to edit the 
 report. This option takes "grade:action" pairs.
 HERE
@@ -276,6 +277,21 @@ HERE
         default => undef,
         validate => \&_validate_grade_action_pair,
     },
+    send_skipfile => {
+        prompt => "What file has patterns for things that shouldn't be reported?",
+        default => undef,
+        validate => \&_validate_skipfile,
+    },
+    cc_skipfile => {
+        prompt => "What file has patterns for things that shouldn't CC to authors?",
+        default => undef,
+        validate => \&_validate_skipfile,
+    },
+    command_timeout => {
+        prompt => "If no timeout is set by CPAN, halt system commands after how many seconds?",
+        default => undef,
+        validate => \&_validate_seconds,
+    },
     email_to => {
         default => undef,
     },
@@ -297,9 +313,14 @@ sub _config_spec { return %option_specs }
 #--------------------------------------------------------------------------#
 
 sub _get_config_dir {
-    return ( $^O eq 'MSWin32' )
-        ? File::Spec->catdir(File::HomeDir->my_documents, ".cpanreporter")
-        : File::Spec->catdir(File::HomeDir->my_home, ".cpanreporter") ;
+    if ( defined $ENV{PERL_CPAN_REPORTER_DIR} ) {
+        return $ENV{PERL_CPAN_REPORTER_DIR};
+    }
+    else {
+        return ( $^O eq 'MSWin32' )
+            ? File::Spec->catdir(File::HomeDir->my_documents, ".cpanreporter")
+            : File::Spec->catdir(File::HomeDir->my_home, ".cpanreporter") ;
+    }
 }
 
 #--------------------------------------------------------------------------#
@@ -307,7 +328,12 @@ sub _get_config_dir {
 #--------------------------------------------------------------------------#
 
 sub _get_config_file {
-    return File::Spec->catdir( _get_config_dir, "config.ini" );
+    if ( defined $ENV{PERL_CPAN_REPORTER_CONFIG} ) {
+        return $ENV{PERL_CPAN_REPORTER_CONFIG};
+    }
+    else {
+        return File::Spec->catdir( _get_config_dir, "config.ini" );
+    }
 }
 
 #--------------------------------------------------------------------------#
@@ -339,6 +365,7 @@ sub _get_config_options {
     return \%active;
 }
 
+
 #--------------------------------------------------------------------------#
 # _grade_action_prompt -- describes grade action pairs
 #--------------------------------------------------------------------------#
@@ -462,6 +489,20 @@ sub _validate_grade_action_pair {
     return scalar(keys %ga_map) ? \%ga_map : undef;
 }
 
+sub _validate_seconds {
+    my ($name, $option) = @_;
+    return unless $option && ($option =~ /^\d/) && $option > 0;
+    return $option;
+}
+
+sub _validate_skipfile {
+    my ($name, $option) = @_;
+    return unless $option;
+    my $skipfile = File::Spec->file_name_is_absolute( $option )
+                 ? $option : File::Spec->catfile( _get_config_dir(), $option );
+    return -r $skipfile ? $skipfile : undef;
+}
+
 1;
 __END__
 
@@ -598,10 +639,21 @@ A better way to disable CPAN::Reporter temporarily is with the CPAN option
 
 These additional options are only necessary in special cases, for example if
 the default editor cannot be found or if reports shouldn't be sent in 
-certain situations.
+certain situations or for automated testing, and so on.
 
-* {cc_author = <grade:action> ...} -- should module authors should be sent a copy of 
-the test report at their {author@cpan.org} address? (default:yes pass/na:no)
+* {cc_author = <grade:action> ...} -- should module authors should be sent a 
+copy of the test report at their {author@cpan.org} address? 
+(default:yes pass/na:no)
+* {cc_skipfile = <skipfile>} -- filename containing regular expressions (one
+per line) to match against the distribution ID (e.g. 
+'AUTHOR/Dist-Name-0.01.tar.gz'); the author will not be copied if a match is 
+found regardless of cc_author; non-absolute filename must be in the .cpanreporter 
+config directory
+* {command_timeout} -- if set and the CPAN config {inactivity_timeout} is not, 
+then any commands executed by CPAN::Reporter will be halted after this many 
+seconds; useful for unattended smoke testing to stop after some amount of time;
+generally, this should be large -- 900 seconds or more -- as some 
+distributions' tests take quite a long time to run
 * {editor = <editor>} -- editor to use to edit the test report; if not set,
 Test::Reporter will use environment variables {VISUAL}, {EDITOR} or {EDIT}
 (in that order) to find an editor 
@@ -613,6 +665,8 @@ reports be sent, regardless of {send_report}? (default:no)
 {send_report} during the make phase
 * {send_test_report = <grade:action> ...} -- if defined, used in place of 
 {send_report} during the test phase
+* {send_skipfile = <skipfile>} -- like {cc_skipfile} but no report will be 
+sent at all if a match is found
 * {transport = <transport>} -- if defined, passed to the {transport()} 
 method of [Test::Reporter].  Valid options are 'Net::SMTP' or 
 'Mail::Send'.  (CPAN::Reporter uses Net::SMTP for this by default.)
@@ -628,9 +682,24 @@ These options are useful for debugging only:
 * {email_to = <email address>} -- alternate destination for reports instead of
 {cpan-testers@perl.org}; used for testing
 
+= ENVIRONMENT
+
+The following environment variables may be set to alter the default locations 
+for CPAN::Reporter files:
+
+* {PERL_CPAN_REPORTER_DIR} -- if set, this directory is used in place of
+the default .cpanreporter directory; this will affect not only the location
+of the default {config.ini}, but also the location of the 
+[CPAN::Reporter::History] database and any other files that live in that
+directory
+* {PERL_CPAN_REPORTER_CONFIG} -- if set, this file is used in place of 
+the default {config.ini} file; it may be in any directory, regardless of the 
+choice of configuration directory
+
 = SEE ALSO
 
 * [CPAN::Reporter]
+* [CPAN::Reporter::History]
 * [CPAN::Reporter::FAQ]
 
 = AUTHOR
@@ -639,7 +708,7 @@ David A. Golden (DAGOLDEN)
 
 = COPYRIGHT AND LICENSE
 
-Copyright (c) 2006, 2007 by David A. Golden
+Copyright (c) 2006, 2007, 2008 by David A. Golden
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
